/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Expiration date for `Cookie`.
 */

var expireDate = new Date(0);

/**
 * Initialize a new `Cookie` with the given `name`, `value`, and `options`.
 *
 * @param {String} name
 * @param {any} value
 * @param {Object} (optional) options
 * @api public
 */

var Cookie = exports.Cookie = function(name, value, options) {
  if (typeof name !== 'string' || !name.length) {
    throw new Error('Invalid cookie name: ' + name);
  }

  Object.defineProperty(this, 'name', { value: name });

  if (typeof value === 'undefined' || value === null) {
    this.value = '';
    this.expire();
  } else {
    this.value = value;
  }

  for (var name in options) {
    this[name] = options[name];
  }
};

/**
 * Prototype.
 */

Cookie.prototype = {
  path: '/',
  expires: undefined,
  domain: undefined,
  secure: false,
  httpOnly: true,

  /**
   * Expire this cookie.
   *
   * @return {Cookie} this `Cookie`
   * @api public
   */

  expire: function() {
    this.expires = expireDate;
    return this;
  },

  /**
   * Convert to HTTP header value.
   *
   * @return {String}
   * @api public
   */

  toString: function() {
    var value = this.value;

    if (typeof this.value === 'string') {
      if (value.indexOf('"') !== -1) {
        value = encodeURIComponent(value);
      } else {
        value = JSON.stringify(value);
      }
    } else {
      value = encodeURIComponent(JSON.stringify(value));
    }

    var header = [this.name + '=' + value];

    if (this.path) header.push('path=' + this.path);
    if (this.expires) header.push('expires=' + this.expires.toUTCString());
    if (this.domain) header.push('domain=' + this.domain);
    if (this.secure) header.push('secure');
    if (this.httpOnly) header.push('httponly');

    header = header.join('; ');

    if (header.length >= 4096) {
      console.warn('Cookie size should be limited to 4096 bytes for cookie ' + this.name);
    }

    return header;
  },

  /**
   * Return a copy of this cookie.
   *
   * @return {Cookie} a copy of this `Cookie`
   * @api public
   */

  copy: function() {
    return new Cookie(this.name, this.value, this);
  }
};
