/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var Cookie = require('./cookie').Cookie;

/**
 * Initialize a new `CookieJar` with the given `cookies` and `options`.
 *
 * @param {Array[Cookie]} (optional) cookies
 * @param {Object} (optional) options
 * @api public
 */

var CookieJar = exports.CookieJar = function(cookies, options) {
  if (arguments.length === 1 && !Array.isArray(cookies)) {
    options = cookies;
    cookies = null;
  }

  this.options = options || {};
  this._cookies = {};

  if (cookies) {
    for (key in cookies) {
      this.set(cookies[key]);
    }
  }
};

/**
 * Prototype.
 */

CookieJar.prototype = {

  /**
   * Get cookie by `name`.
   *
   * @param {String} name
   * @return {Cookie} the cookie
   * @api public
   */

  get: function(name) {
    name = name.toLowerCase();
    return this._cookies[name];
  },

  /**
   * Set cookie `name` to `value` with `options`.
   *
   * @param {String} name
   * @param {any} value
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  set: function(name, value, options) {
    var cookie;

    if (options) {
      for (key in this.options) {
        if (typeof options[key] === 'undefined') {
          options[key] = this.options[key];
        }
      }
    } else {
      options = this.options;
    }

    if (arguments.length === 1) {
      cookie = name;
      name = cookie.name.toLowerCase();
      value = undefined;
      options = undefined;
    } else {
      cookie = new Cookie(name, value, options);
      name = name.toLowerCase();
    }

    this._cookies[name] = cookie;

    return this;
  },

  /**
   * Remove and return cookie `name`.
   *
   * @param {String} name
   * @return {Cookie} the removed cookie
   * @api public
   */

  remove: function(name) {
    name = name.toLowerCase();
    var cookie = this._cookies[name];
    delete this._cookies[name];
    return cookie;
  },

  /**
   * Populate `obj` with an object keyed with the cookie names.
   *
   * @param {Object} obj
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  saveCookieValues: function(obj) {
    var getter, cookie;

    for (name in this._cookies) {
      cookie = this._cookies[name];
      if (getter = cookie.__lookupGetter__('value')) {
        (function(name, getter) {
          Object.defineProperty(obj, name, {
            get: function() {
              delete this[name];
              return this[name] = getter();
            },
            enumerable: true,
            configurable: true
          });
        })(name, getter);
      } else {
        obj[name] = cookie.value;
      }
    }

    return this;
  },

  /**
   * Set `Set-Cookie` header on `res`.
   *
   * Cookies flagged as `secure` will not be set if the connection is not
   * secure.
   *
   * @param {Object} req
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  setHeader: function(req, res) {
    var headers = []
      , socket = req.connection;

    for (name in this._cookies) {
      var cookie = this._cookies[name]
      if (!cookie.secure || socket.encrypted || socket.proxySecure) {
        headers.push(cookie.toString());
      }
    }

    if (headers.length >= 20) {
      console.warn('The number of cookies set should be limited to 20');
    }

    if (headers.length) {
      res.setHeader('Set-Cookie', headers);
    }

    return this;
  }
};
