/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , Cookie = require('../lib/index').Cookie;

/**
 * Vows tests.
 */

vows.describe('Cookie').addBatch({
  'when created without a name': {
    'should throw an error': function() {
      assert.throws(function() { new Cookie(); }, Error);
    }
  },
  'when using default options': {
    topic: function() {
      return new Cookie('Test', 'Hello World!');
    },
    'should be ok': function(topic) {
      assert.equal(topic.path, '/');
      assert.isTrue(topic.httpOnly);
      assert.isUndefined(topic.expires);
      assert.isUndefined(topic.domain);
      assert.isFalse(topic.secure);
    },
    'should set expire() to Date(0)': function(topic) {
      topic.expire();
      assert.deepEqual(topic.expires, new Date(0));
    }
  },
  'when stringifying RFC 2109 Example 1.3': {
    topic: function() {
      return new Cookie('Customer', 'WILE_E_COYOTE', { path: '/acme' });
    },
    'should be ok': function(topic) {
      assert.equal(topic.name, 'Customer');
      assert.equal(topic.value, 'WILE_E_COYOTE');
      assert.equal(topic.path, '/acme');
    }
  },
  'when stringifying RFC 2109 Example 1.5': {
    topic: function() {
      return new Cookie('Part_Number', 'Rocket_Launcher_0001', { path: '/acme' });
    },
    'should be ok': function(topic) {
      assert.equal(topic.name, 'Part_Number');
      assert.equal(topic.value, 'Rocket_Launcher_0001');
      assert.equal(topic.path, '/acme');
    }
  },
  'when stringifying a string value': {
    topic: function() {
      return new Cookie('Test', 'Hello World!');
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test="Hello World!"; path=/; httponly');
    },
  },
  'when stringifying a string value with a quote': {
    topic: function() {
      return new Cookie('Test', '"Hello World!"');
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=%22Hello%20World!%22; path=/; httponly');
    },
  },
  'when stringifying an object': {
    topic: function() {
      return new Cookie('Test', { message: 'Hello World!' });
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=%7B%22message%22%3A%22Hello%20World!%22%7D; path=/; httponly');
    },
  },
  'when stringifying an array': {
    topic: function() {
      return new Cookie('Test', ['Hello World!']);
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=%5B%22Hello%20World!%22%5D; path=/; httponly');
    },
  },
  'when stringifying a number': {
    topic: function() {
      return new Cookie('Test', 42);
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=42; path=/; httponly');
    },
  },
  'when stringifying true': {
    topic: function() {
      return new Cookie('Test', true);
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=true; path=/; httponly');
    },
  },
  'when stringifying false': {
    topic: function() {
      return new Cookie('Test', false);
    },
    'should be ok': function(topic) {
      assert.equal(topic.toString(), 'Test=false; path=/; httponly');
    },
  },
  'when stringifying null': {
    topic: function() {
      return new Cookie('Test', null);
    },
    'should be expiring': function(topic) {
      assert.equal(topic.toString(), 'Test=""; path=/; expires=Thu, 01 Jan 1970 00:00:00 GMT; httponly');
    },
  },
  'when stringifying undefined': {
    topic: function() {
      return new Cookie('Test');
    },
    'should be expiring': function(topic) {
      assert.equal(topic.toString(), 'Test=""; path=/; expires=Thu, 01 Jan 1970 00:00:00 GMT; httponly');
    },
  }
}).export(module);
