/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , EventEmitter = require('events').EventEmitter
  , Cookie = require('../lib/index').Cookie
  , CookieJar = require('../lib/index').CookieJar;

/**
 * Vows tests.
 */

vows.describe('CookieJar').addBatch({
  'when adding cookies from RFC 2109 Example 1.5': {
    topic: function() {
      var jar = new CookieJar();
      jar.set('Customer', 'WILE_E_COYOTE', { path: '/acme' });
      jar.set('Part_Number', 'Rocket_Launcher_0001', { path: '/acme' });
      return jar;
    },
    'should be ok': function(topic) {
      var customer = topic.get('customer')
        , partNumber = topic.get('part_number');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    },
    'should match case-insensitively': function(topic) {
      var customer = topic.get('CUSTOMER')
        , partNumber = topic.get('PART_NUMBER');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    },
    'should save the cookie values in a hash': function(topic) {
      var hash = {};
      topic.saveCookieValues(hash);
      assert.equal(hash.customer, 'WILE_E_COYOTE');
      assert.equal(hash.part_number, 'Rocket_Launcher_0001');
    },
    'should return the cookie when removing': function(topic) {
      var cookie = topic.remove('customer');
      assert.equal(cookie.name, 'Customer');
      assert.equal(cookie.value, 'WILE_E_COYOTE');
      assert.equal(cookie.path, '/acme');
      assert.isUndefined(cookie.expires);
      assert.isUndefined(cookie.domain);
      assert.isFalse(cookie.secure);
      assert.isTrue(cookie.httpOnly);
      assert.isUndefined(topic.get('customer'));
    }
  },
  'when using cookies from RFC 2109 Example 1.5': {
    topic: function() {
      return new CookieJar([
        new Cookie('Customer', 'WILE_E_COYOTE', { path: '/acme' }),
        new Cookie('Part_Number', 'Rocket_Launcher_0001', { path: '/acme' })
      ]);
    },
    'should be ok': function(topic) {
      var customer = topic.get('customer')
        , partNumber = topic.get('part_number');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    },
    'should match case-insensitively': function(topic) {
      var customer = topic.get('CUSTOMER')
        , partNumber = topic.get('PART_NUMBER');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    },
    'should save the cookie values in a hash': function(topic) {
      var hash = {};
      topic.saveCookieValues(hash);
      assert.equal(hash.customer, 'WILE_E_COYOTE');
      assert.equal(hash.part_number, 'Rocket_Launcher_0001');
    },
    'should return the cookie when removing': function(topic) {
      var cookie = topic.remove('customer');
      assert.equal(cookie.name, 'Customer');
      assert.equal(cookie.value, 'WILE_E_COYOTE');
      assert.equal(cookie.path, '/acme');
      assert.isUndefined(cookie.expires);
      assert.isUndefined(cookie.domain);
      assert.isFalse(cookie.secure);
      assert.isTrue(cookie.httpOnly);
      assert.isUndefined(topic.get('customer'));
    }
  },
  'when using default options': {
    topic: function() {
      var jar = new CookieJar({
        path: '/path',
        expires: new Date(100),
        domain: '.example.com',
        secure: true,
        httpOnly: false
      });

      jar.set('Test', 'Hello World!');

      return jar.get('test');
    },
    'should be ok': function(topic) {
      assert.equal(topic.value, 'Hello World!');
      assert.equal(topic.path, '/path');
      assert.deepEqual(topic.expires, new Date(100));
      assert.equal(topic.domain, '.example.com');
      assert.isTrue(topic.secure);
      assert.isFalse(topic.httpOnly);
    }
  },
  'when setting the header': {
    topic: function() {
      var jar = new CookieJar;
      jar.set('Test', 'YES');
      return jar
    },
    'should set the header': function(topic) {
      var mock = mockReqRes(false, false);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.isTrue(mock.calledSetHeader === 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.equal(mock.headerValue, 'Test="YES"; path=/; httponly');
    }
  },
  'when setting the header with a secure cookie': {
    topic: function() {
      var jar = new CookieJar;
      jar.set('Test', 'YES', { secure: true });
      return jar
    },
    'should not set the header when insecure': function(topic) {
      var mock = mockReqRes(false, false);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 0);
    },
    'should set the header when secure': function(topic) {
      var mock = mockReqRes(true, false);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.equal(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
    },
    'should set the header when proxy secure': function(topic) {
      var mock = mockReqRes(false, true);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.equal(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
    },
    'should set the header when secure or proxy secure': function(topic) {
      var mock = mockReqRes(true, true);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.equal(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
    }
  },
  'when setting the header with mixed cookies': {
    topic: function() {
      var jar = new CookieJar;
      jar.set('Test', 'YES', { secure: true });
      jar.set('Test2', 'NO');
      return jar
    },
    'should not set the secure cookie header when insecure': function(topic) {
      var mock = mockReqRes(false, false);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.equal(mock.headerValue, 'Test2="NO"; path=/; httponly');
    },
    'should set both cookie headers when secure': function(topic) {
      var mock = mockReqRes(true, false);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.include(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
      assert.include(mock.headerValue, 'Test2="NO"; path=/; httponly');
    },
    'should set both cookie headers when proxy secure': function(topic) {
      var mock = mockReqRes(false, true);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.equal(mock.headerName, 'Set-Cookie');
      assert.include(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
      assert.include(mock.headerValue, 'Test2="NO"; path=/; httponly');
    },
    'should set both cookie headers when secure or proxy secure': function(topic) {
      var mock = mockReqRes(true, true);

      topic.setHeader(mock.request, mock.response);

      assert.isTrue(mock.gotConnection >= 1);
      assert.strictEqual(mock.calledSetHeader, 1);
      assert.include(mock.headerValue, 'Test="YES"; path=/; secure; httponly');
      assert.include(mock.headerValue, 'Test2="NO"; path=/; httponly');
    }
  }
}).export(module);

function mockReqRes(encrypted, proxyEncrypted) {
  var mock = {
      gotConnection: 0
    , gotEncrypted: 0
    , gotProxySecure: 0
    , calledSetHeader: 0
    , headerName: undefined
    , headerValue: undefined
    , socket: {
        get encrypted() {
          ++mock.gotEncrypted;
          return encrypted;
        },
        get proxySecure() {
          ++mock.gotProxySecure;
          return proxyEncrypted;
        }
      }
    , request: {
        get connection() {
          ++mock.gotConnection;
          return mock.socket;
        }
      }
    , response: {
        setHeader: function(name, value) {
          ++mock.calledSetHeader;
          mock.headerName = name;
          mock.headerValue = value;
        }
      }
  };

  return mock;
}
