/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , CookieJar = require('../../lib/index').CookieJar
  , cookieManager = require('../../lib/index').cookieManager;

/**
 * Vows tests.
 */

vows.describe('CookieManager').addBatch({
  'when used as middleware with RFC 2109 Example 1.3': {
    topic: function() {
      var mock = mockReqRes('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.equal(topic.request.cookies.customer, 'WILE_E_COYOTE');
    }
  },
  'when used as middleware with RFC 2109 Example 1.5': {
    topic: function() {
      var mock = mockReqRes('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"; Part_Number="Rocket_Launcher_0001"; $Path="/acme"')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.equal(topic.request.cookies.customer, 'WILE_E_COYOTE');
      assert.equal(topic.request.cookies.part_number, 'Rocket_Launcher_0001');
    },
    'should call writeHead()': function(topic) {
      var writeHeadStatus = Math.random()
        , writeHeadHeaders = Math.random();
      topic.response.writeHead(writeHeadStatus, writeHeadHeaders);
      assert.strictEqual(topic.writeHeadStatus, writeHeadStatus);
      assert.strictEqual(topic.writeHeadHeaders, writeHeadHeaders);
      assert.strictEqual(topic.calledWriteHead, 1);
    }
  },
  'when used as middleware with no cookie': {
    topic: function() {
      var mock = mockReqRes('')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.deepEqual(topic.request.cookies, {});
      assert.isTrue(topic.response.cookies instanceof CookieJar);
    }
  },
  'when using options': {
    topic: function() {
      var mock = mockReqRes('');

      mock.options = {
        path: '/path',
        expires: new Date(100),
        domain: '.example.com',
        secure: true,
        httpOnly: false
      };

      var middleware = cookieManager(mock.options);
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.deepEqual(topic.request.cookies, {});
      assert.isTrue(topic.response.cookies instanceof CookieJar);
    },
    'should pass through options to CookieJar': function(topic) {
      assert.deepEqual(topic.response.cookies.options, topic.options);
    }
  }
}).export(module);

function mockReqRes(cookieHeader) {
  var mock = {
      gotHeaders: 0
    , gotCookie: 0
    , calledNext: 0
    , calledWriteHead: 0
    , writeHeadStatus: undefined
    , writeHeadHeaders: undefined
    , setHeaderName: undefined
    , setHeaderValue: undefined
    , headers: {
        get cookie() {
          ++mock.gotCookie;
          return cookieHeader;
        }
      }
    , request: {
        get headers() {
          ++mock.gotHeaders;
          return mock.headers;
        }
      }
    , response: {
        setHeader: function(name, value) {
          ++mock.calledSetHeader;
          mock.setHeaderName = name;
          mock.setHeaderValue = value;
        }
      , writeHead: function(status, headers) {
          ++mock.calledWriteHead;
          mock.writeHeadStatus = status;
          mock.writeHeadHeaders = headers;
        }
      }
    , next: function() {
        ++mock.calledNext;
      }
  };

  return mock;
}
